﻿namespace MobileOSGeekAudioPlaybackAgent;

interface

uses
  System,
  System.Diagnostics,
  System.Windows,
  Microsoft.Phone.BackgroundAudio;

type
  AudioPlayer = public class(AudioPlayerAgent)

  private
    /// <remarks>
    /// AudioPlayer instances can share the same process.
    /// Static fields can be used to share state between AudioPlayer instances
    /// or to communicate with the Audio Streaming agent.
    /// </remarks>
    class constructor ;

    /// Code to execute on Unhandled Exceptions
    class method UnhandledException(sender: System.Object; e: ApplicationUnhandledExceptionEventArgs);
    
    /// <summary>
    /// Implements the logic to get the next AudioTrack instance.
    /// In a playlist, the source can be from a file, a web request, etc.
    /// </summary>
    /// <remarks>
    /// The AudioTrack URI determines the source, which can be:
    /// (a) Isolated-storage file (Relative URI, represents path in the isolated storage)
    /// (b) HTTP URL (absolute URI)
    /// (c) MediaStreamSource (null)
    /// </remarks>
    /// <returns>an instance of AudioTrack, or null if the playback is completed</returns>
    method GetNextTrack: AudioTrack;

    /// <summary>
    /// Implements the logic to get the previous AudioTrack instance.
    /// </summary>
    /// <remarks>
    /// The AudioTrack URI determines the source, which can be:
    /// (a) Isolated-storage file (Relative URI, represents path in the isolated storage)
    /// (b) HTTP URL (absolute URI)
    /// (c) MediaStreamSource (null)
    /// </remarks>
    /// <returns>an instance of AudioTrack, or null if previous track is not allowed</returns>
    method GetPreviousTrack: AudioTrack;

  protected
    /// <summary>
    /// Called when the playstate changes, except for the Error state (see OnError)
    /// </summary>
    /// <param name="player">The BackgroundAudioPlayer</param>
    /// <param name="track">The track playing at the time the playstate changed</param>
    /// <param name="playState">The new playstate of the player</param>
    /// <remarks>
    /// Play State changes cannot be cancelled. They are raised even if the application
    /// caused the state change itself, assuming the application has opted-in to the callback.
    ///
    /// Notable playstate events:
    /// (a) TrackEnded: invoked when the player has no current track. The agent can set the next track.
    /// (b) TrackReady: an audio track has been set and it is now ready for playack.
    ///
    /// Call NotifyComplete() only once, after the agent request has been completed, including async callbacks.
    /// </remarks>
    method OnPlayStateChanged(player: BackgroundAudioPlayer; track: AudioTrack; playState: PlayState); override;

    /// <summary>
    /// Called when the user requests an action using application/system provided UI
    /// </summary>
    /// <param name="player">The BackgroundAudioPlayer</param>
    /// <param name="track">The track playing at the time of the user action</param>
    /// <param name="action">The action the user has requested</param>
    /// <param name="param">The data associated with the requested action.
    /// In the current version this parameter is only for use with the Seek action,
    /// to indicate the requested position of an audio track</param>
    /// <remarks>
    /// User actions do not automatically make any changes in system state; the agent is responsible
    /// for carrying out the user actions if they are supported.
    ///
    /// Call NotifyComplete() only once, after the agent request has been completed, including async callbacks.
    /// </remarks>
    method OnUserAction(player: BackgroundAudioPlayer; track: AudioTrack; action: UserAction; param: System.Object); override;
    
    /// <summary>
    /// Called whenever there is an error with playback, such as an AudioTrack not downloading correctly
    /// </summary>
    /// <param name="player">The BackgroundAudioPlayer</param>
    /// <param name="track">The track that had the error</param>
    /// <param name="error">The error that occured</param>
    /// <param name="isFatal">If true, playback cannot continue and playback of the track will stop</param>
    /// <remarks>
    /// This method is not guaranteed to be called in all cases. For example, if the background agent
    /// itself has an unhandled exception, it won't get called back to handle its own errors.
    /// </remarks>
    method OnError(player: BackgroundAudioPlayer; track: AudioTrack; error: Exception; isFatal: System.Boolean); override;

    /// <summary>
    /// Called when the agent request is getting cancelled
    /// </summary>
    /// <remarks>
    /// Once the request is Cancelled, the agent gets 5 seconds to finish its work,
    /// by calling NotifyComplete()/Abort().
    /// </remarks>
    method OnCancel; override;
  end;

implementation

class constructor AudioPlayer;
begin
  // Subscribe to the managed exception handler
  Deployment.Current.Dispatcher.BeginInvoke(method begin
    Application.Current.UnhandledException += UnhandledException
  end)

end;

class method AudioPlayer.UnhandledException(sender: System.Object; e: ApplicationUnhandledExceptionEventArgs);
begin
  if Debugger.IsAttached then begin
    // An unhandled exception has occurred; break into the debugger
    Debugger.Break()
  end
end;

method AudioPlayer.OnPlayStateChanged(player: BackgroundAudioPlayer; track: AudioTrack; playState: PlayState);
begin
    case playState of 
    PlayState.TrackEnded: begin
      player.Track := GetPreviousTrack();
    end;
    PlayState.TrackReady: begin
      player.Play();
    end;

    PlayState.Shutdown: 
    // TODO: Handle the shutdown state here (e.g. save state) 
	;
    PlayState.Unknown: ;

    PlayState.Stopped: ;

    PlayState.Paused: ;

    PlayState.Playing: ;

    PlayState.BufferingStarted: ;

    PlayState.BufferingStopped: ;

    PlayState.Rewinding: ;

    PlayState.FastForwarding: ;
  end;

  NotifyComplete()
end;

method AudioPlayer.OnUserAction(player: BackgroundAudioPlayer; track: AudioTrack; action: UserAction; param: System.Object);
begin
  case action of 
    UserAction.Play: begin
      if player.PlayerState <> PlayState.Playing then begin
        player.Play()
      end;
    end;
    UserAction.Stop: begin
      player.Stop();
    end;

    UserAction.Pause: begin
      player.Pause();
    end;
    UserAction.FastForward: begin
      player.FastForward();
    end;
    UserAction.Rewind: begin
      player.Rewind();
    end;
    UserAction.Seek: begin
      player.Position := TimeSpan(param);
    end;
    UserAction.SkipNext: begin
      player.Track := GetNextTrack();
    end;
    UserAction.SkipPrevious: begin
      var previousTrack: AudioTrack := GetPreviousTrack();
      if previousTrack <> nil then begin
        player.Track := previousTrack
      end;
    end;  end;

  NotifyComplete()
end;

method AudioPlayer.GetNextTrack: AudioTrack;
begin
  // TODO: add logic to get the next audio track
  var track: AudioTrack := nil;

  // specify the track
  exit track
end;

method AudioPlayer.GetPreviousTrack: AudioTrack;
begin
  // TODO: add logic to get the previous audio track
  var track: AudioTrack := nil;

  // specify the track
  exit track
end;

method AudioPlayer.OnError(player: BackgroundAudioPlayer; track: AudioTrack; error: Exception; isFatal: System.Boolean);
begin
  if isFatal then begin
    Abort()
  end
  else begin
    NotifyComplete()
  end
end;

method AudioPlayer.OnCancel;
begin

end;

end.
